;-------------------------------------------------------------
; JL rules
; Basic concept: want to follow Zoran's convention for 
;	(Bx, By, Bz) and (xarr, yarr, zarr) not Metcalf's.
; (The latter provides a smart handling of coordiantes but we want
; higher resolution in intense magnetic field region, so...)
;--- in specific:
; 1. [xarr,yarr,zarr] will be in pixel units
;    but they not in uniform grid nut adaptive grid
; 2. For this reason, we do not use Metcalf routines 
;    or IDL built-in fcn (bilinear.pro) because it assumes
;    uniform grid. So use dblin.pro for interpolating (unfortunately).
; 3. (xarr, yarr, zarr) always start from (0,0,0) and all positive
;     max(xarr) >= max(obs_xarr) & max(yarr) >= max(obs_yarr)
;	due to projection effect
;-------------------------------------------------------------

	innam='afff.out'
	outnam='fldext.out'

print,'get ',innam

restore,innam

nx=n_elements(xarr)
ny=n_elements(yarr)
nz=n_elements(zarr)
nl=n_elements(xbeg)
Bz0=reform(Bz(*,*,0),nx,ny)
xl=fltarr(nl,20) & yl=xl & zl=xl & Bl=xl

xmax=max(xarr) & xmin=min(xarr)
ymax=max(yarr) & ymin=min(yarr)
zmax=max(zarr) & zmin=min(zarr)

;---- start

window,0,xs=600,ys=600
contour,Bz0,xarr,yarr,xst=1,yst=1,nlev=14,/fill 

ds=1.0
ns=10001

for l=0,nl-1 do begin

if (Bz(xbeg(l),ybeg(l),0) lt 0) then begin
	print,'Change from +ve to -ve polarity region'
        Bx = -Bx & By = -By & Bz = -Bz
endif


xs=fltarr(ns) & ys=fltarr(ns) & zs=fltarr(ns)
Bxs=fltarr(ns) & Bys=fltarr(ns) & Bzs=fltarr(ns) & BTs=Bxs
Fxy=fltarr(nx,ny)

; + + + + + + determining the starting point s_0   + + + + + +
u=xarr(xbeg(l))
v=yarr(ybeg(l))
oplot,[u],[v],psym=1
xs(0)=u 
ys(0)=v
zs(0)=0.

Bxs(0)= Bx(xbeg(l),ybeg(l),0)
Bys(0)= By(xbeg(l),ybeg(l),0)
Bzs(0)= Bz(xbeg(l),ybeg(l),0)
Bts(0)=sqrt(Bxs(0)^2+Bys(0)^2+Bzs(0)^2)

; the first integration using Euler's formula - - - - - - - -
xs(1)=xs(0)+ds*Bxs(0)/Bts(0)
ys(1)=ys(0)+ds*Bys(0)/Bts(0)
zs(1)=zs(0)+ds*Bzs(0)/Bts(0)

; + + + + + + determining the Bx, By, Bz at s_1   + + + + + +
i=0
u=xs(1)
v=ys(1)
w=zs(1)
ip=min(where(xarr ge u))
jp=min(where(yarr ge v))
kp=min(where(zarr ge w))
xr =(u-xarr(ip-1))/(xarr(ip)-xarr(ip-1))	
yr =(v-yarr(jp-1))/(yarr(jp)-yarr(jp-1))
zr =(w-zarr(kp-1))/(zarr(kp)-zarr(kp-1))

fxy(*,*)=Bx(*,*,kp-1)
Fz1=fdblin(fxy,ip,jp,xr,yr)
fxy(*,*)=Bx(*,*,kp)
Fz2=fdblin(fxy,ip,jp,xr,yr)
Bxs(1)= Fz1 + (Fz2-Fz1)*zr 

fxy(*,*)=By(*,*,kp-1)
Fz1=fdblin(fxy,ip,jp,xr,yr)
fxy(*,*)=By(*,*,kp)
Fz2=fdblin(fxy,ip,jp,xr,yr)
Bys(1)= Fz1 + (Fz2-Fz1)*zr

fxy(*,*)=Bz(*,*,kp-1)
Fz1=fdblin(fxy,ip,jp,xr,yr)
fxy(*,*)=Bz(*,*,kp)
Fz2=fdblin(fxy,ip,jp,xr,yr)
Bzs(1)= Fz1 + (Fz2-Fz1)*zr

Bts(1)=sqrt(Bxs(1)^2+Bys(1)^2+Bzs(1)^2)
; + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + 

; - - - - - next integration using Adams's formula - - - - - - 
for i=1,ns-2 do begin
;print,i+1
xs(i+1)=xs(i) + 1.5*ds*Bxs(i)/Bts(i) -.5*ds*Bxs(i-1)/Bts(i-1)
ys(i+1)=ys(i) + 1.5*ds*Bys(i)/Bts(i) -.5*ds*Bys(i-1)/Bts(i-1)
zs(i+1)=zs(i) + 1.5*ds*Bzs(i)/Bts(i) -.5*ds*Bzs(i-1)/Bts(i-1)

if (xs(i+1) ge xmax) or (xs(i+1) le xmin) or (ys(i+1) ge ymax) $
 or (ys(i+1) le ymin) or (zs(i+1) ge zmax)or (BTs(i) eq 0) then goto, hitbound
if (zs(i+1) le 0.0) then goto, hitground

; + + + + + + determining the Bx, By, Bz at s_{i+1} + + + + + +

u=xs(i+1)
v=ys(i+1)
w=zs(i+1)
ip=min(where(xarr ge u))
jp=min(where(yarr ge v))
kp=min(where(zarr ge w))
xr =(u-xarr(ip-1))/(xarr(ip)-xarr(ip-1))	
yr =(v-yarr(jp-1))/(yarr(jp)-yarr(jp-1))
zr =(w-zarr(kp-1))/(zarr(kp)-zarr(kp-1))
fxy(*,*)=Bx(*,*,kp-1)
Fz1=fdblin(fxy,ip,jp,xr,yr)
fxy(*,*)=Bx(*,*,kp)
Fz2=fdblin(fxy,ip,jp,xr,yr)
Bxs(i+1)=Fz1 + (Fz2-Fz1)*zr

fxy(*,*)=By(*,*,kp-1)
Fz1=fdblin(fxy,ip,jp,xr,yr)
fxy(*,*)=By(*,*,kp)
Fz2=fdblin(fxy,ip,jp,xr,yr)
Bys(i+1)=Fz1 + (Fz2-Fz1)*zr

fxy(*,*)=Bz(*,*,kp-1)
Fz1=fdblin(fxy,ip,jp,xr,yr)
fxy(*,*)=Bz(*,*,kp)
Fz2=fdblin(fxy,ip,jp,xr,yr)
Bzs(i+1)=Fz1 + (Fz2-Fz1)*zr

Bts(i+1)=sqrt(Bxs(i+1)^2+Bys(i+1)^2+Bzs(i+1)^2)

endfor

goto,skk

hitbound: 
print,'Line no.',l,' got out of the box'
imax=i
sx= xs(0:imax)
sy= ys(0:imax)
sz= zs(0:imax)
st=Bts(0:imax)

xwrite=congrid(sx,20,interp=1)
ywrite=congrid(sy,20,interp=1)
zwrite=congrid(sz,20,interp=1)
Bwrite=congrid(st,20,interp=1)


goto,skk

hitground: imax=i+1
if imax gt ns-1 then imax=ns-1
sx= xs(0:imax)
sy= ys(0:imax)
sz= zs(0:imax)
st=Bts(0:imax)

xwrite=congrid(sx,20,interp=1)
ywrite=congrid(sy,20,interp=1)
zwrite=congrid(sz,20,interp=1)
Bwrite=congrid(st,20,interp=1)

xwrite=fltarr(20)
ywrite=fltarr(20)
zwrite=fltarr(20)
Bwrite=fltarr(20)

xwrite(0:18)=congrid(sx,19,interp=1)
ywrite(0:18)=congrid(sy,19,interp=1)
zwrite(0:18)=congrid(sz,19,interp=1)
Bwrite(0:18)=congrid(st,19,interp=1)

ratio=sz(imax-1)/(sz(imax-1)-sz(imax))
xwrite(19)=sx(imax-1)*(1.-ratio) +sx(imax)*ratio
ywrite(19)=sy(imax-1)*(1.-ratio) +sy(imax)*ratio
zwrite(19)=sz(imax-1)*(1.-ratio) +sz(imax)*ratio
Bwrite(19)=st(imax-1)


skk:
oplot,xwrite,ywrite,thick=1

xl(l,*)=xwrite
yl(l,*)=ywrite
zl(l,*)=zwrite
Bl(l,*)=Bwrite

endfor

print,'file being written: ',outnam
save,filename='fldext.out',xl,yl,zl,bl,Bz0,xarr,yarr
end
