;-------------------------------
; read the extrapolation results
;-------------------------------

restore,'reduced.sav'
window,0,xs=600,ys=600
contour,Bz(*,*,0),xarr,yarr,xst=1,yst=1,nlev=14,/fill 

nl=55
xl=fltarr(20,nl-1)
yl=xl & zl=xl & Bl=xl
openr,1,'fldext.out'
hama=fltarr(20)
for l=0,nl-2 do begin
readu,1,hama & xl(*,l)=hama(*)
readu,1,hama & yl(*,l)=hama(*)
readu,1,hama & zl(*,l)=hama(*)
readu,1,hama & Bl(*,l)=hama(*)
endfor
close,1

for l=0,nl-2 do oplot,xl(*,l),yl(*,l)

outfile='afff.out'

;----------------------------------
; read original (obs) data 
;----------------------------------
; should reuse the parameteres used in afff.pro here again
Rsun=948.63179		; a real number in units of [arcsec] (from Almanac)
AR_box=[1280,870,1280+255,870+255]	; [i1,j1,i2,j2] ; coordinate of the subfield
BL=a(AR_box(0):AR_box(2), AR_box(1):AR_box(3)) ; extrat the active region
; position information of the target active region (read from the image file)
CX_pixel=float(sxpar(HDR,'E_XCEN')) 
CY_pixel=float(sxpar(HDR,'E_YCEN'))
pixelsiz=[1.0,1.0]*sxpar(HDR,'SCALE')
THETA= asin( (0.5*(AR_box(2)+AR_box(0)) - CX_pixel)*pixelsiz(0)/Rsun )
PHIEE= asin( (0.5*(AR_box(3)+AR_box(1)) - CY_pixel)*pixelsiz(1)/Rsun )
; check
; contour,a ; full image
; Rpix=Rsun/pixelsiz(0)
; oplot,[cx],[cy],psym=1,symsize=5.0 ; plots solar disk center
; oplot,[cx,cx-Rpix],[cy,cy]	; solar x radius
; oplot,[cx,cx],[cy,cy-Rpix]   	; solar y radius

;*****************************************
; transform them back
;*****************************************

 p =30.*!dtor		; This is so because we want to rotate the sunspot
 b0=0.	 	; to the apparent disk center.
 lc= THETA 	; in [rad] - see above
 bc= PHIEE 

 p  = -p
 b0 = -b0
 lc = -lc
 bc = -bc

axx = - SIN(b0)*SIN(p)*SIN(lc) + COS(p)*COS(lc)
axy = + SIN(b0)*COS(p)*SIN(lc) + SIN(p)*COS(lc)
axz = - COS(b0)*SIN(lc)
ayx = - SIN(bc)*( SIN(b0)*SIN(p)*COS(lc) + COS(p)*SIN(lc) ) $
      - COS(bc)*COS(b0)*SIN(p)
ayy = + SIN(bc)*( SIN(b0)*COS(p)*COS(lc) - SIN(p)*SIN(lc) ) $
      + COS(bc)*COS(b0)*COS(p)
ayz = - COS(b0)*SIN(bc)*COS(lc) + SIN(b0)*COS(bc)
azx = + COS(bc)*( SIN(b0)*SIN(p)*COS(lc) + COS(p)*SIN(lc) ) $
      - SIN(bc)*COS(b0)*SIN(p)
azy = - COS(bc)*( SIN(b0)*COS(p)*COS(lc) - SIN(p)*SIN(lc) ) $
      + SIN(bc)*COS(b0)*COS(p)
azz = + COS(bc)*COS(b0)*COS(lc) + SIN(bc)*SIN(b0)

; These are the transformation matrix elements given by eq. (2)
; of Gary and Hagyard, Solar Phys. 126, 21 (1990).
; The image-plane coordinates xI are related to the heliographic coordinates
; xH by xI = C xH, where C = [cxx cxy]
;			     [cyx cyy]
; THIS ASSUMES zH=0 I.E. ON PHOTOSPHERE, so 2x2 matrix instead of 3x3.
; Note that zI = czx*xH + czy*yH, which they do not define, but
; it exists (and is nonzero).

cxx = axx & cxy = ayx & cyx = axy & cyy = ayy

; keep these for reference
;xi = pixelsiz(0)*FINDGEN(mx)#REPLICATE(1,my) 
;yi = pixelsiz(1)*REPLICATE(1,mx)#FINDGEN(my)
;zi = - ( azx*xi + azy*yi )/azz ; "height" of photosphere along LOS
;xh = axx*xi + axy*yi + axz*zi 
;yh = ayx*xi + ayy*yi + ayz*zi
;zh = azx*xi + azy*yi + azz*zi ; this should be all zero (only checking purpose)

; what we want is
xh=xarr
yh=yarr
zh=0.
xi = axx*xh + axy*yh + axz*zh 
yi = ayx*xh + ayy*yh + ayz*zh
zi = azx*xh + azy*yh + azz*zh

print,' regriding'

triangulate,xh,yh,tr

nx=128 & ny=128 
ps=[min(xh),min(yh),max(xh),max(yh)]
gs=[(ps(2)-ps(0))/(nx-1),(ps(3)-ps(1))/(ny-1)]
;zmax=max([abs(ps(2)-ps(0)),abs(ps(3)-ps(1))])*0.3
zmax=100. ; arcsec
;xn=gs(0)*FINDGEN(nx)#REPLICATE(1,ny)
;yn=gs(1)*REPLICATE(1,nx)#FINDGEN(ny)
xn=gs(0)*FINDGEN(nx)
yn=gs(1)*FINDGEN(ny)
zn=[findgen(10)*0.01,0.1+findgen(16)*0.02, $
	0.42,0.45,0.48,0.52,0.56,0.6,0.65,0.7,0.75,0.8,0.88,1.0]*zmax
nz=n_elements(zn)
Bz0=trigrid(xh,yh,Bz0,tr,gs,ps)
; this is the Bz (z=0) in the heliocentric (or apparent disk center)
x1d=0. & y1d=0. & B1d=0.
; now want to get B s.t.
;------------------------------------
    B = FLTARR(Nx,Ny,Nz,3,/NOZERO)
;------------------------------------
fa = FFT(Bz0,-1) ; Fourier representation of B_n (not B_l)
fa(0,0) = 0.0	; make sure net flux is zero

; heliocentric spatial frequencies (k_x,k_y,k_z) in  [radians per arcsec]
kx = 2*!PI/Nx*SHIFT(INDGEN(Nx)-(Nx-1)/2,-(Nx-1)/2)#REPLICATE(1,Ny)/gs(0)
ky = 2*!PI/Ny*REPLICATE(1,Nx)#SHIFT(INDGEN(Ny)-(Ny-1)/2,-(Ny-1)/2)/gs(1)
kz2= kx^2 + ky^2

alpha = 0. 
alpha2 = complex(alpha^2,0.)

k = sqrt(complex(kz2,0.0)-alpha2)
kl2 = COMPLEX(0.0, kz2)
kl2(0,0) = 1.0  ; (0,0) is singular, dont divide by zero

print,'Compute heliographic field'

iphihat0 = fa/kl2

FOR iz=0,Nz-1 DO BEGIN
print,zn(iz)
  iphihat = iphihat0*EXP(-k*zn(iz)) ; get field at height z
  fbx = (k*kx-alpha*ky)*iphihat
  fby = (k*ky+alpha*kx)*iphihat
  fbz = complex(0.,1.)*(kz2)*iphihat
  B(*,*,iz,0) = FLOAT(FFT(fbx,1))
  B(*,*,iz,1) = FLOAT(FFT(fby,1))
  B(*,*,iz,2) = FLOAT(FFT(fbz,1))
ENDFOR

print,'successful end. Results are saved to ',outfile
save,filename=outfile,B,xn,yn,zn
; return,B

END
