PRO YSETUP,BTR,BLG,F2GYRO,YTDEN,YTNUM,RTFAC,FNUM,DPOL
;----------------------------------------------------------------------------
; Calculates correction to free-free opacity for non-zero magnetic field.
;----------------------------------------------------------------------------
   YT2=BTR*BTR/F2GYRO
   YL2=BLG*BLG/F2GYRO
   yl = abs(blg/sqrt(f2gyro))
   YT4 = YT2*YT2
   YTDEN = 2. - YT2
   YTNUM = 4. + 2.*YT2
   RTFAC = SQRT(YT4 + 4.*YL2)
   bad = where(RTFAC eq 0,nbad)
   if (nbad ne 0) then RTFAC[bad] = .0001*YTDEN[bad]

   ; r0 is the ellipticity of the o-mode electric vector

   r0 = (-1./yl)*(yt2 - rtfac)
   dpol = (blg/abs(blg))*r0/(1.+r0*r0)
   FNUM = 2.*YT4/RTFAC
RETURN
END

FUNCTION DTAUF,ED,TE,DZ,F2
;---------------------------------------------------------------------------
; This function calculates free-free opacity in absence of magnetic field,
; based on formula from Dulk (1985).  Note that DZ is in km, F in Hz.
; 10% He is assumed, so Z_i^2 n_i = 1.17 n_e.
;---------------------------------------------------------------------------
   low_t = where(TE le 2.e5,nlow_t)
   high_t = where(TE gt 2.e5,nhigh_t)
   C = TE  ; Just declares C same size as TE, contents to be overridden
   if (nlow_t ne 0) then C[low_t] = 18.2 + 1.5*ALOG(TE[low_t]) - ALOG(ED[low_t])
   if (nhigh_t ne 0) then C[high_t] = 24.5 + ALOG(TE[high_t]) - ALOG(ED[high_t])

RETURN,(1.14e3*ED*ED*C*DZ)/(F2*(TE^1.5))
END

PRO GSETUP,BTR,BT,IS,F1,F2,F3,F4,F5
;----------------------------------------------------------------------------
;   Calculates the terms that make up the angular dependence of the gyro-
;   resonance opacity.  Note that IS is one less than the harmonic number
;   (IS = s - 1) and that ISN is the harmonic number.  This angular dependence
;   only needs to be calculated for integral f/fB.
;----------------------------------------------------------------------------
   S4 = [4.,16.,36.,64.,100.,144.,196.,256.,324.,400.]
   SINB=BTR/BT
   SINB2=SINB*SINB
   COSB2=1.-SINB2
   F1=SINB2^IS
   F2=SINB2+2.*COSB2*(IS+1.0)
   F4=SINB2*SINB2+S4[IS]*COSB2
   F3=SQRT(F4)
   F5=SINB2*F3
RETURN
END

FUNCTION GFNC,ED,TE,DZ,FGHZ,BT0,BT,IS
;----------------------------------------------------------------------------
;   This function returns the gyro-resonance opacity that is to be modulated
;   by the angular dependent terms WORD and WEXT in main program.  Note that
;   the variable IS is one less than the harmonic number (IS = s - 1) and
;   ISN is the harmonic number.
;----------------------------------------------------------------------------
   GFAC=[1332.,5.308E3,4.031E4,4.53E5,6.749E6,1.254E8,2.79E9,$
         7.237E10,2.144E12,7.142E13]

RETURN,(GFAC[IS]*ED*DZ*(1.6864E-10*TE)^IS)/(FGHZ*(1.-BT0/BT))
END

FUNCTION SIS1,BTR,BT,ED,TE,DZ,FGHZ,BT0
;----------------------------------------------------------------------------
;   This function returns the gyro-resonance opacity for o-mode for the
;   special case s = 1 (IS = 0).
;----------------------------------------------------------------------------
   SINB=BTR/BT
   SINB2=SINB*SINB
   COSB2=1.-SINB2
   RES=(1.0614E4*ED*DZ*(1.6864E-10*TE)^2)/(FGHZ*(1.-BT0/BT)) $
        *(COSB2*SINB2*SINB2)/(2.*(1. + COSB2))
RETURN,RES
END

pro mcmem,model,fghz, BR, BL

   F2GHZ  = FGHZ*FGHZ
   FGYRO  = FGHZ/2.79922E6
   F2GYRO = FGYRO*FGYRO
   SEX = 0
   SOR = 0
   low = where(model.bt le 0.1,nlow)
   if (nlow ne 0) then model.bt[low]=0.1
   NZ = n_elements(model.z)
   NX = n_elements(model.x)
   NY = n_elements(model.y)
   BR  = fltarr(nx,ny)
   BL  = fltarr(nx,ny)
   TAUOR = fltarr(nx,ny)
   TAUEX = fltarr(nx,ny)
   TFFOR = fltarr(nx,ny)
   TFFEX = fltarr(nx,ny)
   BTR = SQRT(model.bt^2 - model.bz^2) ;SQRT(model.bx^2+model.by^2)
   BLG = model.bz
   BT = model.bt ; Total field strength
   TE = model.te
   ED = model.dn

   ; Get vertical step size in cm, from model Z values.  DZ[0] is undefined,
   ; so set it to something reasonable, e.g. = DZ[1]
   DZ = (model.z - shift(model.z,1));*1.e5
   DZ[0] = DZ[1]
   DL = TE  ; Just declares L same size as TE, contents to be overridden
   for i = 0, n_elements(dz)-1 do begin
      DL[*,*,i] = DZ[i]
   endfor

   ISPREV = INTARR(NX,NY)*0 + 10

   YSETUP,BTR,BLG,F2GYRO,YTDEN,YTNUM,RTFAC,FNUM,DPOL

   DTAU = DTAUF(ED,TE,DL,F2GHZ)

; Calculate gyroresonance and free-free opacity in both modes, starting
; at top of model and proceeding downward.

   BT01 = BT[*,*,nz-1]
   for iz = nz-1,0,-1 do begin
      DTOR = fltarr(nx,ny)
      DTEX = fltarr(nx,ny)
      DTFOR=DTAU[*,*,iz]*(YTNUM[*,*,iz]-FNUM[*,*,iz])$
                        /(YTDEN[*,*,iz]+RTFAC[*,*,iz])^2
      TFFOR = TFFOR + DTFOR
      DTFEX=DTAU[*,*,iz]*(YTNUM[*,*,iz]+FNUM[*,*,iz])$
                        /(YTDEN[*,*,iz]-RTFAC[*,*,iz])^2
      TFFEX = TFFEX + DTFEX
      IS = LONG(FGYRO/BT[*,*,iz])  ; Harmonic numbers for this height level
      ; Get 2D versions of arrays, for this height level
      BTR1 = BTR[*,*,iz]
      BT1  =  BT[*,*,iz]
      ED1  =  ED[*,*,iz]
      TE1  =  TE[*,*,iz]
      DL1  =  DL[*,*,iz]

      good = where(IS lt ISPREV,ngood)
      if (ngood ne 0) then begin
         s1 = where(IS[good] eq 0,ns1)    ; Points where s=1 (IS = 0)
         sg1 = where(IS[good] gt 0,nsg1)  ; Points where s>1 (IS > 0)
         if (ns1 ne 0) then begin

            ; IS = 0 means that the harmonic number s = 1.  This requires special
            ; treatment since x-mode reflects at this layer and o-mode opacity
            ; is given by a different equation than GSETUP and GFNC.  At present
            ; no reflection at x-mode is allowed for s = 1.

			; Get 1D version of arrays, for only s=1 points in layer
            BTR2 = BTR1[s1]
            BT2  =  BT1[s1]
            ED2  =  ED1[s1]
            TE2  =  TE1[s1]
            DL2  =  DL1[s1]
            BT02 = BT01[s1]
            DTOR[s1] = SIS1(BTR2,BT2,ED2,TE2,DL2,FGHZ,BT02)
            DTEX[s1] = 0.0
		 endif else if (nsg1 ne 0) then begin

            ; IS > 0 means that harmonic number s > 1, so do the
            ; typical calculation for these grid points

			; Get 1D version of arrays, for only s>1 points in layer
            BTR2 = BTR1[sg1]
            BT2  =  BT1[sg1]
            IS2  =   IS[sg1]
            ED2  =  ED1[sg1]
            TE2  =  TE1[sg1]
            DL2  =  DL1[sg1]
            BT02 = BT01[sg1]
            ; Calculate angular terms WORD and WEXT
            GSETUP,BTR2,BT2,IS2,F1,F2,F3,F4,F5

            WEXT = ((F2 + F3)^2)*F1/(F4 + F5)
            WORD = ((F2 - F3)^2)*F1/(F4 - F5)
            bad = where(F4 EQ F5,nbad)
            if (nbad ne 0) then WORD[bad] = 0.0

            GF = GFNC(ED2,TE2,DL2,FGHZ,BT02,BT2,IS2)
            bad = where(is2 ge 10.,nbad)
            if (nbad ne 0) then GF[bad] = 0.0

            DTOR[sg1] = GF*WORD
            DTEX[sg1] = GF*WEXT
         endif
      endif
;C
;C   If the optical depth has reached unity in the corona (TE .gt. 2.e5),
;C   save the harmonic number in SEX and SOR (these are the highest optically
;C   thick harmonics in the two modes).
;C
;		  IF(SEX.EQ.0.AND.DTEX.GT.1..AND.TE.GE.2.E5)
;     *			SEX = IS + 1
;		  IF(SOR.EQ.0.AND.DTOR.GT.1..AND.TE.GE.2.E5)
;     *			SOR = IS + 1
;
      ; Increment optical depths for the contribution from this layer
      TAUOR = TAUOR + DTOR
      TAUEX = TAUEX + DTEX

      ; Determine increment to brightness temperature from this layer
      DBTOR=TE*EXP(-(DTOR+DTFOR))*EXP(-(TAUOR+TFFOR-DTOR-DTFOR))
      DBTEX=TE*EXP(-(DTEX+DTFEX))*EXP(-(TAUEX+TFFEX-DTEX-DTFEX))

      ; Having determined the brightness in o- and x-mode, convert to
      ; brightness in RH and LH circular polarization.

      BR = BR + 0.5*(DBTOR + DBTEX) + DPOL*(DBTOR - DBTEX)
	  BL = BL + 0.5*(DBTOR + DBTEX) - DPOL*(DBTOR - DBTEX)
      good = where(IS le 9,ngood)
      if (ngood gt 0) then ISPREV[good] = IS[good]
      BT01=BT1
   endfor

   ; We have done all height layers, and now BR and BL contain the final maps.
return
end