;+
; NAME:
;     mw_gr_layers
; PURPOSE:
;     calculates the opacities due to gyroresonant absorption
;	upto harmponic=6
; 1.determines the k-index in floating number (upto n=6) which is
;   later used for interpolation purposes.
; 2. Bt in [gauss], frq in [GHz], ns = max harmonic in your mind + 1
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_gr_layers, Bt, fGHz, ns
; INPUTS:
;     Bt	Magnetic field strength Array in [gauss]
;     fGHz      frequency array in [GHz]
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
; ROUTINES CALLED:
; OUTPUTS:
;     fk	indicating the locations of the gyroresonant layers
;		(from s=1 to s=6) in the index space of z-array.
; COMMENTS:
;     1. fk(harmonic,frequency,x,y) = -1 signals the absence of
;	     the corresponding harmonic layer
;     2. As a safe-guard for a case of multiple layers for one harmonic,
;     	 i.e. when mag. field does NOT monotonically decrease outward,
;        pick the highest layer only.
; SIDE EFFECTS:
;     Throw out the lowest and the highest layers from consideration.
; RESTRICTIONS:
;     Still unsure if it handles well a case of NON-monotonic vari..of B
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;     09-Mar-2003  DG
;       Changed to put fastest varying indexes first to avoid maniacal
;       page faulting.
;-

function mw_gr_layers, Bt, fGHz

nx = n_elements(Bt[*,0,0])
ny = n_elements(Bt[0,*,0])
nz = n_elements(Bt[0,0,*])
nf = n_elements(fGHz)
ns = 7 ; should coicide with the max. harmonic in mw_gr_opacity.pro

float_k=fltarr(nf,ns,nx,ny) ; final output
zindx=findgen(nz) 	; used as reference

for j=0,ny-1 do begin
for i=0,nx-1 do begin

 B1  =reform(Bt[i,j,*])
 Bmin=min(B1) & Bmax=max(B1)

 for q=0,nf-1 do begin
  float_k[q,*,i,j] = -1.0

  for s=1,ns-1 do begin
   Bs = fGHz[q]*1.0E9/(2.8E6*float(s))
   kun=max(where(B1 ge Bs)) ; see the above NB
   if (kun gt 1) and (kun lt (nz-2)) then $
   float_k[q,s,i,j] = kun + (B1[kun]-Bs)/(B1[kun]-B1[kun+1])
  endfor ; next harminic

 endfor ; next frequency

endfor ; next x
endfor ; next y

return, float_k
end
