;+
; NAME:
;     mw_gr_plus_ff_rl
; PURPOSE:
;     insert the gyro-resonant layers into the original Te3.
;
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_gr_plus_ff_rl, ff, gr, fk
; INPUTS:
;     ff   	structure containing free-free stuff
;     gr       	structure containing gyro-resn stuff
;     fk	array indicating locations of the harmon.layers
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
; ROUTINES CALLED:
; OUTPUTS:
;     gf	structure containing the total opacity
; COMMENTS:
;     take ff as reference and then modify them by inserting gr into ..
; SIDE EFFECTS:
; RESTRICTIONS:
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;     09-Mar-2003  DG
;       Changed to put fastest varying indexes first to avoid maniacal
;       page faulting.
;     06-Apr-2003  DG
;       Changed to calculate R, L instead of X, O.
;-

function mw_gr_plus_ff_rl, ff, gr, fk

Te3f  =ff.te
tau3r =ff.tau_r
tau3l =ff.tau_l

nx =n_elements(Te3f[0,0,*,0])
ny =n_elements(Te3f[0,0,0,*])
nz =n_elements(Te3f[0,*,0,0])
nf =n_elements(Te3f[*,0,0,0])
ns =n_elements(gr.Te[0,*,0,0])

  for j=0,ny-1 do begin
  for i=0,nx-1 do begin

   for q=0,nf-1 do begin

 	 Te  = reform( te3f[q,*,i,j])
	 epr = reform(tau3r[q,*,i,j])
     epl = reform(tau3l[q,*,i,j])

   ; now the goal is to replace some points in (Te, epn)
   ; with those matching nearby gr-layers & of course add the gr-opacity

     for s=1, ns-1 do begin
 	    k1 = fix(fk[q,s,i,j])

   	    if k1 gt 0 then begin ; No gyroresonace layer, No change to f-f table
 	       fr = fk[q,s,i,j] - k1

	       ; interpolate temperature
 	       Te3f[q,k1,i,j] =  Te[k1]*(1.-fr) + Te[k1+1]*fr
	       ;=gr.Te[i,j,s,q], but had better do it agin in some case...

 	       ; interpolate opacity in x-mode (idea: tau_ff reduced a bit,
           ; instead get tau_gr while the layer underneath extends a bit up.

           tau3r[q,k1  ,i,j]=epr[k1  ]*(1.-fr)+ gr.tau_r[q,s,i,j]
           tau3r[q,k1-1,i,j]=epr[k1-1]        + epr[k1]*fr

	       ; interpolate opacity in o-mode

           tau3l[q,k1  ,i,j]=epl[k1  ]*(1.-fr)+ gr.tau_l[q,s,i,j]
           tau3l[q,k1-1,i,j]=epl[k1-1]        + epl[k1]*fr

        endif

     endfor

   endfor ; next frequency

  endfor ; next j
  endfor ; next i

gf = {Te: Te3f, tau_r:tau3r, tau_l:tau3l}

return,gf

end
