
PRO mw_modlin,hint,outfile=outfile

;+
; NAME:
;     mw_modlin
; PURPOSE:
;     create input to the main program, mwr.pro
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mwr[,outfile=outfile]
; INPUTS:
;     Currently, only an hint is needed, i.e. whether 'pot' or 'ff',
;     which of course depends on the file name (and directory).
;     But many things should be specitied within this code such as:
;     the files containing Bx,By,Bz,Te,Dn, along with the physical
;     scales for these quantities. Should contact original modeller.
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;     outf	outfile name
; ROUTINES CALLED:
;     get_hdf (in case the original model is given in .hdf format)
; OUTPUTS:
;     a structure called "model"
; COMMENTS:
;     We have so far coorperated with Zoran Mikic and Y. Mok.
;     They provide model in form of hdf files for tem, rho, Bx, By, Bz.
;     Each file contains (x,y,z) in adaptive grids. Each time ask them
;     to have the same x,y,z for all quantities, and the scales for
;     all these quantities. In this routine, the physical scales are
;     applied, and let this be known to all the rest routines.
;     e.g.,
;     The current model used below is from Y. Mok, and I give
;     some scales arbitrary: I found the model has
;     -0.6< x <0.2, -0.3< y <0.3, 0< z <0.3.
;     Assuming whole AR extent is 40" wide ~ 0.4 in xscale
;     x=0.1->10", x=1->100"-> 7.25E7 * 100 = 7.25E9
;     These are abitrarily chosen at present.
; SIDE EFFECTS:
;     In case authors unspecify x,y,z scales, you may want to
;     try your own, which will then cause the opacity to vary by
;     a constant factor.
; RESTRICTIONS:
;     1. Should contact the modeller to get correct information,
;     2. Need to make file-specific changes every time.
;     3. Assumed that (x,y,z) are same all quantities.
;     4. The scales defined here should be known to all other routines.
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;-

if NOT keyword_set(outfile) then outfile='mw_modlin.sav'

infiles=hint+'_'+['bx','by','bz','tem','rho']+'.hdf'

print,' Reading files: '
for k=0,n_elements(infiles)-1 do print,'   ',infiles[k]
get_hdf,infiles[0],Bx,x,y,z
get_hdf,infiles[1],By,x,y,z
get_hdf,infiles[2],Bz,x,y,z
get_hdf,infiles[3],Te,x,y,z
get_hdf,infiles[4],Dn,x,y,z

Print,' Set scales:'
xscale_arcs=100. & print,' 	x = 1 -> 100 arcsec'
Bscale = 22. 	 & print,'	B = 1 -> 22  Gauss'
tscale=2.8E7 	 & print,'      Te= 1 -> 2.8E7 K '
dscale=3.0E8 	 & print,'    rho = 1 -> 3.0E8 cm-3'
print,' Contact the modeller if unsure of the above values.'

bx = bx * Bscale
by = by * Bscale
bz = bz * Bscale
Te = Te * Tscale
Dn = Dn * dscale
x = x * xscale_arcs
y = y * xscale_arcs
z = z * xscale_arcs

model = {bx:bx, by:by, bz:bz, te:te, Dn:Dn, x:x, y:y, z:z}

print,' Saving the model to: ',outfile
save,filename=outfile,model

print,'All done. Type IDL> mw to proceed.'

end
