;+
; NAME:
;     mw_plot
; PURPOSE:
;     main program for calculating brightness temperature maps
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_plot [,infile=infile][,xr=xr][,yr=yr]
; INPUTS:
;     infile    output of mwr.pro
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;	xr	two-element array
;	yr	two-element array
; ROUTINES CALLED:
; OUTPUTS:
;     	mw_maps.ps
; COMMENTS:
;       input contains TB0,TB1,TB2,x,y,fGHz,infile
;       TB=[x,y,fre]-> real=x-mode, imagin=o-mode
;
; SIDE EFFECTS:
; RESTRICTIONS:
;	Very restrictive at the moment---plots the maps for
;       the first & the last frequencies only in two columns
;	and three cases (ff, gr, both) in rows on one page.
;       Need to make this routine more capable or try you own.
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)

pro mw_plot,infile=infile,xr=xr,yr=yr

if NOT keyword_set(infile) then infile='mw_maps.sav'
restore,infile

if NOT keyword_set(xr) then xr=[min(x),max(x)]
if NOT keyword_set(yr) then yr=[min(y),max(y)]
nf=n_elements(fGHz)	; for purpose of plotting

set_plot,'ps'
device,filename='idl.ps',landscape=0
device,xoff=2.0,yoff=3.0,xs=19,ys=25 ; ,/color,bits_per_pixel=8
!p.font=0
!p.multi=[0,2,3]
!p.charsize=1.2

xs=8000 & xs1=9500
ys=6000 & ys1=7000

xpos=[0,1]*xs1
ypos=[2,1,0]*ys1

hama=fltarr(101,101) & hama[0,0]=-1
ilev=exp(alog(0.1)+alog(1.0/0.1)*findgen(9)/8.)
mlev=[-0.5,-0.3,-0.2,-0.1,-0.05]/10.
plev=[0.05, 0.1, 0.2, 0.3, 0.5]/10.
hint=['f-f only','g-r only','both']

for q=0,2 do begin

if q eq 0 then TB=TB0
if q eq 1 then TB=TB1
if q eq 1 then TB=TB2

y0=ypos[q] ; Total I

for k=0,nf-1,nf-1 do begin

x0=xpos[k<1]
 i=float(TB[*,*,k])+imaginary(TB[*,*,k])
 v=float(TB[*,*,k])-imaginary(TB[*,*,k])
 imax=max(i)
 tvscl,hama,x0,y0,xs=xs,ys=ys,/dev
 contour,Bz0,x,y,nlev=12,/fill,xst=1,yst=1, $
	pos=[x0,y0,x0+xs,y0+ys],/dev,/noera,$
	xtit='x [arcsec]',ytit='y [arcsec]'
 contour,i,x,y,xr=xr,yr=yr,xst=5,yst=5, $
	pos=[x0,y0,x0+xs,y0+ys],/dev,/noera, $
	lev=imax*ilev,c_thick=2, c_color=[100]
 contour,v,x,y,xr=xr,yr=yr,xst=5,yst=5, $
	pos=[x0,y0,x0+xs,y0+ys],/dev, /noera, $
	lev=imax*plev
 contour,v,x,y,xr=xr,yr=yr,xst=5,yst=5, $
	pos=[x0,y0,x0+xs,y0+ys],/dev, /noera, $
	lev=imax*mlev,c_li=1
endfor

endfor

for k=0,nf-1,nf-1 do begin
x0=xpos[k<1]+xs/2.1 
fGHz_c=strmid(strcompress(string(fGHz[k]),/remove_all),0,3)

 for q=0,2 do begin
  cap=fGHz_c+'GHz, '+hint[q]
  y0=ypos[q]+300
  xyouts,x0,y0,cap,/dev,charsize=1.2
 endfor

endfor

x0=0 & y0=3*ys1+300
xyouts,x0,y0,'Microwave Tb maps from: '+infile,/dev
device,/close
set_plot,'x'

end
