;+
; NAME:
;     mw_gr_plus_ff
; PURPOSE:
;     insert the gyro-resonant layers into the original Te3.
;
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mw_gr_plus_ff, ff, gr, fk
; INPUTS:
;     all   structure containing all the quantity need for calculating
;	    brightness temperature. This means (Te, dtau) at each (x,y,z,f)
;	    After integral along z (LOS), we end up with TB(x,f,f)
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
; ROUTINES CALLED:
; OUTPUTS:
;     TB	Brightness temperature on (x,y) plane and frequency
; COMMENTS:
; 1. integration start from the sun rather than from the earth
; 2. intregartion is replaced with iterative relationship
; in this case \tau is only incremental \tau from the previous layer
; SIDE EFFECTS:
;	Returning vale, TB(x,f,f) is an complex arr to accomo. polaiz.
; RESTRICTIONS:
;     Assume that the lowest layer is entirey optically thick while
;     the highest layer is entirely thin. (implied by for k=1,nz-2...)
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;     09-Mar-2003  DG
;       Changed to put fastest varying indexes first to avoid maniacal
;       page faulting.
;-

function mw_soltb, all

nx =n_elements(all.Te[0,0,*,0])
ny =n_elements(all.Te[0,0,0,*])
nz =n_elements(all.Te[0,*,0,0])
nf =n_elements(all.Te[*,0,0,0])

TB_x=fltarr(nf,nx,ny)
TB_o=fltarr(nf,nx,ny)


for q=0,nf-1 do begin

  TB2d = reform(all.Te[q,0,*,*],nx,ny)
  for k=1,nz-2 do begin
    Te2d = reform(all.Te[q,k,*,*],nx,ny)
 	ep2d = 1.0/exp(reform(all.tau_x[q,k,*,*],nx,ny)<66.)
 	TB2d = TB2d*ep2d + Te2d*(1.0 - ep2d )
  endfor
  TB_x[q,*,*] = TB2d

  TB2d = reform(all.Te[q,0,*,*],nx,ny)
  for k=1,nz-2 do begin
 	Te2d = reform(all.Te[q,k,*,*],nx,ny)
 	ep2d = 1.0/exp(reform(all.tau_o[q,k,*,*],nx,ny)<66.)
 	TB2d = TB2d*ep2d + Te2d*(1.0 - ep2d )
  endfor
  TB_o[q,*,*] = TB2d

endfor ; next frequency

return,complex(TB_x,TB_o)

end
