;function fdblin,fxy,ip,jp,xr,yr
;f2=(fxy(ip,jp-1) -fxy(ip-1,jp-1))*xr+fxy(ip-1,jp-1)
;f1=(fxy(ip, jp ) -fxy(ip-1, jp) )*xr+fxy(ip-1, jp)
;ans=(f1-f2)*yr + f2
;return,ans
;end

PRO xtrapol,hint,startxy=startxy,outfile=outfile

;+
; NAME:
;     mw_modlin
; PURPOSE:
;     create input to the main program, mwr.pro
; CATEGORY:
;     Scientific simulation for FASR?
; CALLING SEQUENCE:
;     mwr[,outfile=outfile]
; INPUTS:
;     Currently, only an hint is needed, i.e. whether 'pot' or 'ff',
;     which of course depends on the file name (and directory).
;     But many things should be specitied within this code such as:
;     the files containing Bx,By,Bz,Te,Dn, along with the physical
;     scales for these quantities. Should contact original modeller.
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;     outf	outfile name
; ROUTINES CALLED:
;     get_hdf (in case the original model is given in .hdf format)
; OUTPUTS:
;     a structure called "model"
; COMMENTS:
;     We have so far coorperated with Zoran Mikic and Y. Mok.
;     They provide model in form of hdf files for tem, rho, Bx, By, Bz.
;     Each file contains (x,y,z) in adaptive grids. Each time ask them
;     to have the same x,y,z for all quantities, and the scales for
;     all these quantities. In this routine, the physical scales are
;     applied, and let this be known to all the rest routines.
;     e.g.,
;     The current model used below is from Y. Mok, and I give
;     some scales arbitrary: I found the model has
;     -0.6< x <0.2, -0.3< y <0.3, 0< z <0.3.
;     Assuming whole AR extent is 40" wide ~ 0.4 in xscale
;     x=0.1->10", x=1->100"-> 7.25E7 * 100 = 7.25E9
;     These are abitrarily chosen at present.
; SIDE EFFECTS:
;     In case authors unspecify x,y,z scales, you may want to
;     try your own, which will then cause the opacity to vary by
;     a constant factor.
; RESTRICTIONS:
;     1. Should contact the modeller to get correct information,
;     2. Need to make file-specific changes every time.
;     3. Assumed that (x,y,z) are same all quantities.
;     4. The scales defined here should be known to all other routines.
; MODIFICATION HISTORY:
;     Written 11-Nov-2002 by Jeongwoo Lee (JL)
;-

;============ input ===================

;infiles=hint+'_'+['bx','by','bz']+'.hdf'

print,' Reading files: '
restore,'mw_modlin.sav'
bx = model.bx
by = model.by
bz = model.bz
xarr = model.x
yarr = model.y
zarr = model.z

;for k=0,n_elements(infiles)-1 do print,'   ',infiles[k]
;get_hdf,infiles[0],Bx,xarr,yarr,zarr
;get_hdf,infiles[1],By,xarr,yarr,zarr
;get_hdf,infiles[2],Bz,xarr,yarr,zarr

;xarr=xarr/max(xarr)*100.
;yarr=yarr/max(yarr)*100.
;zarr=zarr/max(zarr)*100.

;============set parameters used in computation======
nx=n_elements(xarr)
ny=n_elements(yarr)
nz=n_elements(zarr)

B3D=fltarr(nx,ny,nz,3)

B3D[*,*,*,0]=Bx
B3D[*,*,*,1]=By
B3D[*,*,*,2]=Bz

if keyword_set(startxy) then begin
   ibeg=reform(startxy[*,0])
   jbeg=reform(startxy[*,1])
endif else begin
   bt0 = reform(sqrt(model.bx[*,*,0]^2+model.by[*,*,0]^2 $
                    +model.bz[*,*,0]^2))
   bmax = max(bt0)
   bint = bmax/30.
   blim = bmax-bint
   blah = reform(indgen(nx*ny),nx,ny)
   npt = 5
   ibeg = intarr(npt*30)
   jbeg = intarr(npt*30)
   k = 0
   for i = 0, 29 do begin
      blim = blim - bint
      good = where(bt0 gt blim,ngood)
      idx = fix(randomu(s,npt)*ngood)>0<ngood
      ridx = (blah[good])[idx]
      ibeg[k:k+npt-1] = blah[ridx] mod nx
      jbeg[k:k+npt-1] = blah[ridx]/nx
      k = k + npt
   endfor
   ;iham=[-4,-2,-1,1, 2, 4]*(nx/10) + nx/2
   ;ibeg=[iham,iham,iham]
   ;jbeg=[iham*0+ny/2,iham*0+ny/2+2,iham*0+ny/2-2]

   ;iham=fltarr(11,7)
   ;jham=fltarr(11,7)
   ;for i=0,10 do iham[i,*]=fix(nx/2 + (nx/13.)*(i-5))
   ;for j=0,6  do jham[*,j]=fix(ny/2 + (ny/10.)*(j-3))
   ;ibeg=reform(iham,77)
   ;jbeg=reform(jham,77)
endelse

; check if Bz=0
print,'any problem with starting points?',where(Bz[ibeg,jbeg,0] eq 0.)


nl=n_elements(ibeg)
Bz0=reform(Bz[*,*,0],nx,ny)

xmax=max(xarr) & xmin=min(xarr)
ymax=max(yarr) & ymin=min(yarr)
zmax=max(zarr) & zmin=min(zarr)

lm=sqrt((xmax-xmin)^2+(ymax-ymin)^2+(zmax-zmin)^2)
ns=2001
ds=lm/float(ns-1)

;============ output ======
; final product you want
xl=fltarr(nl,20) & yl=xl & zl=xl & Bl=xl
if NOT keyword_set(outfile) then outfile='fldlines.sav'

;=========== run ============
rat=(xmax-xmin)/(ymax-ymin)
window,0,xs=800,ys=800/rat
contour,Bz0,xarr,yarr,xst=1,yst=1,nlev=14,/fill

for l=0,nl-1 do begin

ss=fltarr(ns,3)
Bs=fltarr(ns,3)

restart:

i0=ibeg[l] & j0=jbeg[l]
if (B3d[i0,j0,0,2] lt 0.) then B3d=-B3d


;+ + + Advance to the first next point integration using Euler's formula + + +
u=xarr[i0] & v=yarr[j0] & w=0.
ss[0,*]=[u,v,w]  & oplot,[u],[v],psym=1
Bs[0,*]=reform(B3D[i0,j0,0,*])
ss[1,*]=[u,v,w]+ds*reform(Bs[0,*])/sqrt(total(Bs[0,*]^2))
print,'ss0=',ss[0,*]
print,'bs0=',Bs[0,*]
print,'ss1=',ss[1,*]
;+ + + Determining (Bx, By, Bz) at the point by double linear interpolation + +
u=ss[1,0] & v=ss[1,1] & w=ss[1,2]
ip=min(where(xarr ge u))
jp=min(where(yarr ge v))
kp=min(where(zarr ge w))
if kp lt 1 then begin
 ds=ds*2. & print,'too small',ds
 goto,restart
endif
if kp gt 3 then begin
 ds=ds/2. & print,'too large',ds
 goto,restart
endif

xr =(u-xarr[ip-1])/(xarr[ip]-xarr[ip-1])
yr =(v-yarr[jp-1])/(yarr[jp]-yarr[jp-1])
zr =(w-zarr[kp-1])/(zarr[kp]-zarr[kp-1])

for indx=0,2 do begin
Fz1=fdblin(reform(B3d[*,*,kp-1,indx]),ip,jp,xr,yr)
Fz2=fdblin(reform(B3d[*,*,kp,indx]),ip,jp,xr,yr)
Bs[1,indx]= Fz1 + (Fz2-Fz1)*zr
endfor


; + + + From this point on, use Adams's formula for integration + + + + + + +

for p=1,ns-2 do begin

 Bp= reform(Bs[p,*])
 Bq= reform(Bs[p-1,*])

 ss[p+1,*]=reform(ss[p,*]) + 1.5*ds*Bp/sqrt(total(Bp^2)) $
		-.5*ds*Bq/sqrt(total(Bq^2))

 u=ss[p+1,0] & v=ss[p+1,1] & w=ss[p+1,2]
  ; check if integration should be terminated----
  if (u ge xmax) or (u le xmin) or (v ge ymax) or (v le ymin) $
   or (w ge zmax)or (w eq 0) then goto, hitbound
  if (w le 0.0) then goto, hitground
  ; otherwise continue determine B at the point

 ip =min(where(xarr ge u))
 jp =min(where(yarr ge v))
 kp =min(where(zarr ge w))
 xr =(u-xarr(ip-1))/(xarr(ip)-xarr(ip-1))
 yr =(v-yarr(jp-1))/(yarr(jp)-yarr(jp-1))
 zr =(w-zarr(kp-1))/(zarr(kp)-zarr(kp-1))

 for indx=0,2 do begin
 Fz1=fdblin(reform(B3d[*,*,kp-1,indx]),ip,jp,xr,yr)
 Fz2=fdblin(reform(B3d[*,*,kp,indx]),ip,jp,xr,yr)
 Bs[p+1,indx]= Fz1 + (Fz2-Fz1)*zr
 endfor

endfor ; next s

hitbound:
print,'Line no.',l,' got out of the box'
pend=p
sx= reform(ss[0:pend, 0])
sy= reform(ss[0:pend, 1])
sz= reform(ss[0:pend, 2])
Bt= fltarr(pend+1)
for p=0,pend do Bt[p]=sqrt(total(Bs[p,*]^2))

xn=congrid(sx,20,interp=1)
yn=congrid(sy,20,interp=1)
zn=congrid(sz,20,interp=1)
Bn=congrid(Bt,20,interp=1)

goto,skk

hitground: pend=(p+1)<(ns-1)
sx= reform(ss[0:pend, 0])
sy= reform(ss[0:pend, 1])
sz= reform(ss[0:pend, 2])
Bt= fltarr(pend+1)
for p=0,pend do Bt[p]=sqrt(total(Bs[p,*]^2))

xn=(yn=(zn=(Bn=fltarr(20))))
; replace the last one with
xn(0:18)=congrid(sx[0:pend-1],19,interp=1)
yn(0:18)=congrid(sy[0:pend-1],19,interp=1)
zn(0:18)=congrid(sz[0:pend-1],19,interp=1)
Bn(0:18)=congrid(Bt[0:pend-1],19,interp=1)
; put the last one with
ratio=sz[pend-1]/(sz[pend-1]-sz[pend])
xn[19]=sx[pend-1]*(1.-ratio) +sx[pend]*ratio
yn[19]=sy[pend-1]*(1.-ratio) +sy[pend]*ratio
zn[19]=sz[pend-1]*(1.-ratio) +sz[pend]*ratio
Bn[19]=Bt[pend-1]*(1.-ratio) +Bt[pend]*ratio

skk:
oplot,xn,yn,thick=1

xl(l,*)=xn & yl(l,*)=yn & zl(l,*)=zn & Bl(l,*)=Bn

endfor ; next line

print,'file being written: ',outfile
save,filename=outfile,xl,yl,zl,bl,Bz0,xarr,yarr
end

